"""Class implementation of integer.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._type.hashable_interface import HashableInterface
from apysc._type.number_value_mixin import NumberValueMixIn
from apysc._type.to_fixed_mixin import ToFixedMixIn
from apysc._type.to_hex_mixin import ToHexMixIn
from apysc._type.to_number_mixin import ToNumberMixIn
from apysc._type.to_string_mixin import ToStringMixIn
from apysc._validation import arg_validation_decos


class Int(
    NumberValueMixIn[int, "Int"],
    InitializeWithBaseValueInterface,
    ToStringMixIn,
    ToFixedMixIn,
    HashableInterface,
    ToNumberMixIn,
    ToHexMixIn,
):
    """
    Integer class for the apysc library.

    References
    ----------
    - Int and Number
        - https://simon-ritchie.github.io/apysc/en/int_and_number.html
    - Int and Number common arithmetic operations
        - https://simon-ritchie.github.io/apysc/en/int_and_number_arithmetic_operations.html  # noqa
    - Int and Number common comparison operations
        - https://simon-ritchie.github.io/apysc/en/int_and_number_comparison_operations.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> _ = ap.Stage()
    >>> int_val: ap.Int = ap.Int(10)
    >>> int_val
    Int(10)

    >>> int_val == 10
    Boolean(True)

    >>> int_val == ap.Int(10)
    Boolean(True)

    >>> int_val >= 10
    Boolean(True)

    >>> int_val += 10
    >>> int_val
    Int(20)

    >>> int_val = ap.Int(10.5)
    >>> int_val
    Int(10)
    """

    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        value: Union[int, float, NumberValueMixIn],
        *,
        variable_name_suffix: str = "",
        skip_init_substitution_expression_appending: bool = False,
    ) -> None:
        """
        Integer class for apysc library.

        Parameters
        ----------
        value : int or float or Int or Number
            Initial integer value. If the `float` or `Number`
            value is specified, this class casts it to an integer.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        skip_init_substitution_expression_appending : bool, default False
            A boolean indicates whether to skip an initial substitution
            expression or not. This class uses this option internally.

        References
        ----------
        - Int and Number
            - https://simon-ritchie.github.io/apysc/en/int_and_number.html
        - Int and Number common arithmetic operations
            - https://simon-ritchie.github.io/apysc/en/int_and_number_arithmetic_operations.html  # noqa
        - Int and Number common comparison operations
            - https://simon-ritchie.github.io/apysc/en/int_and_number_comparison_operations.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> int_val: ap.Int = ap.Int(10)
        >>> int_val
        Int(10)

        >>> int_val == 10
        Boolean(True)

        >>> int_val == ap.Int(10)
        Boolean(True)

        >>> int_val >= 10
        Boolean(True)

        >>> int_val += 10
        >>> int_val
        Int(20)

        >>> int_val = ap.Int(10.5)
        >>> int_val
        Int(10)
        """
        from apysc._converter import cast
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope
        from apysc._type import type_util

        with TemporaryNotHandlerScope():
            is_number_specified: bool = type_util.is_number(value=value)
            TYPE_NAME: str = var_names.INT
            self.variable_name = expression_variables_util.get_next_variable_name(
                type_name=TYPE_NAME
            )
            super(Int, self).__init__(
                value=value,
                type_name=TYPE_NAME,
                variable_name_suffix=variable_name_suffix,
            )
            self._value = cast.to_int_from_float(int_or_float=self.value)
            self._append_constructor_expression()
            self._append_cast_expression(is_number_specified=is_number_specified)

        self._append_initial_substitution_expression_if_in_handler_scope(
            skip_appending=skip_init_substitution_expression_appending,
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_cast_expression(self, *, is_number_specified: bool) -> None:
        """
        Append integer cast (Math.trunc) expression.

        Parameters
        ----------
        is_number_specified : bool
            Boolean value whether a specified value is Number
            instance or not.
        """
        from apysc._expression import expression_data_util

        if not is_number_specified:
            return
        expression: str = (
            f"{self.variable_name} = Math.trunc({self.variable_name}, 10);"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _set_value_and_skip_expression_appending(
        self, *, value: Union[int, float, NumberValueMixIn]
    ) -> None:
        """
        Update value attribute and skip expression appending.

        Parameters
        ----------
        value : int or float or Int or Number
            Any number value to set. This interface casts that
            value to an integer if float or number value is specified.
        """
        from apysc._converter import cast

        if isinstance(value, NumberValueMixIn):
            value._value = cast.to_int_from_float(int_or_float=value._value)
            value_: Union[int, float, NumberValueMixIn] = value._value
        else:
            value = cast.to_int_from_float(int_or_float=value)
            value_ = value
        self._value = value_  # type: ignore

    def __repr__(self) -> str:
        """
        Get a representation string of this instance.

        Returns
        -------
        repr_str : str
            Representation string of this instance.
        """
        if not hasattr(self, "_value"):
            repr_str: str = "Int(0)"
        else:
            repr_str = f"Int({self._value})"
        return repr_str

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "Int":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        int_value : Int
            An initialized integer value.
        """
        return Int(0)

    @final
    def __hash__(self) -> int:
        """
        Get a hashed value.

        Returns
        -------
        hashed_value : int
            A hashed value.
        """
        return int(self._value)



"""Class implementation for fill alpha mix-in.
"""

from typing import Dict
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._animation.animation_fill_alpha_mixin import AnimationFillAlphaMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class FillAlphaMixIn(
    VariableNameSuffixAttrOrVarMixIn,
    AnimationFillAlphaMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):
    _fill_alpha: Number

    @final
    def _initialize_fill_alpha_if_not_initialized(self) -> None:
        """
        Initialize _fill_alpha attribute if this interface does not
        initialize it yet.
        """
        if hasattr(self, "_fill_alpha"):
            return
        suffix: str = self._get_attr_or_variable_name_suffix(
            value_identifier="fill_alpha"
        )
        self._fill_alpha = Number(
            1.0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_fill_alpha_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_fill_alpha_attr_linking_setting(self) -> None:
        """
        Append a scale-y attribute linking settings.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._fill_alpha, attr_name="fill_alpha"
        )
        self._append_attr_to_linking_stack(
            attr=self._fill_alpha, attr_name="fill_alpha"
        )

    @property
    @add_debug_info_setting(module_name=__name__)
    def fill_alpha(self) -> Number:
        """
        Get this instance's fill opacity.

        Returns
        -------
        fill_alpha : Number
            Current fill opacity (0.0 to 1.0).

        References
        ----------
        - GraphicsBase fill_alpha interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_fill_alpha.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> rectangle.fill_alpha = ap.Number(0.5)
        >>> rectangle.fill_alpha
        Number(0.5)
        """
        from apysc._type import value_util

        self._initialize_fill_alpha_if_not_initialized()
        fill_alpha: Number = value_util.get_copy(value=self._fill_alpha)
        return fill_alpha

    @fill_alpha.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def fill_alpha(self, value: Number) -> None:
        """
        Update this instance's fill opacity.

        Parameters
        ----------
        value : Number
            Fill opacity to set.

        References
        ----------
        - GraphicsBase fill_alpha interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_fill_alpha.html  # noqa
        """
        self._update_fill_alpha_and_skip_appending_exp(value=value)
        self._fill_alpha._append_incremental_calc_substitution_expression()
        self._append_fill_alpha_update_expression()

        self._append_fill_alpha_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_fill_alpha_update_expression(self) -> None:
        """
        Append the fill alpha updating expression.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        value_str: str = value_util.get_value_str_for_expression(value=self._fill_alpha)
        expression: str = f"{self.variable_name}.fill({{opacity: {value_str}}});"
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _update_fill_alpha_and_skip_appending_exp(
        self, *, value: Union[float, Number]
    ) -> None:
        """
        Update the fill opacity and skip appending expression.

        Parameters
        ----------
        value : float or Number
            Fill opacity to set.
        """
        from apysc._converter import cast

        self._initialize_fill_alpha_if_not_initialized()
        if not isinstance(value, Number):
            suffix: str = self._get_attr_or_variable_name_suffix(
                value_identifier="fill_alpha"
            )
            value = cast.to_float_from_int(int_or_float=value)
            value_: Number = Number(value=value, variable_name_suffix=suffix)
        else:
            value_ = value
        self._fill_alpha = value_

    _fill_alpha_snapshots: Optional[Dict[str, float]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_fill_alpha_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_fill_alpha_snapshots",
            value=self._fill_alpha._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._fill_alpha._value = self._get_snapshot_val_if_exists(
            current_value=self._fill_alpha._value,
            snapshot_dict=self._fill_alpha_snapshots,
            snapshot_name=snapshot_name,
        )



"""The loop implementation class for the `ap.Dictionary` values.
"""

from typing import Any
from typing import Dict
from typing import Generic
from typing import Optional
from typing import Type
from typing import TypeVar

from typing_extensions import final

from apysc._expression.get_last_scope_interface import GetLastScopeInterface
from apysc._expression.indent_num import Indent
from apysc._expression.last_scope import LastScope
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.for_loop_exit_mixin import ForLoopExitMixIn
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._type.dictionary import Dictionary
from apysc._type.initialize_locals_and_globals_mixin import (
    InitializeLocalsAndGlobalsMixIn,
)
from apysc._type.string import String
from apysc._validation import arg_validation_decos

_DictValue = TypeVar("_DictValue", bound=InitializeWithBaseValueInterface)


class ForDictValues(
    ForLoopExitMixIn,
    GetLastScopeInterface,
    InitializeLocalsAndGlobalsMixIn,
    Generic[_DictValue],
):
    """
    The loop implementation class for the `ap.Dictionary` values.

    References
    ----------
    - ForDictValues class
        - https://simon-ritchie.github.io/apysc/en/for_dict_values.html

    References
    ----------
    >>> import apysc as ap

    >>> _ = ap.Stage(
    ...     stage_width=250,
    ...     stage_height=150,
    ...     background_color=ap.Color("#333"),
    ...     stage_elem_id="stage",
    ... )
    >>> dict_: ap.Dictionary[str, ap.Number] = ap.Dictionary(
    ...     {"a": ap.Number(50), "b": ap.Number(150)},
    ... )
    >>> with ap.ForDictValues(dict_=dict_, dict_value_type=ap.Number) as value:
    ...     _ = ap.Rectangle(
    ...         x=value, y=50, width=50, height=50, fill_color=ap.Color("#0af")
    ...     )
    ...
    """

    _dict: Dictionary[Any, _DictValue]
    _dict_value_type: Type[_DictValue]
    _variable_name_suffix: str

    @final
    @arg_validation_decos.is_apysc_dict(arg_position_index=1)
    @arg_validation_decos.is_initialize_with_base_value_interface_subclass(
        arg_position_index=2,
    )
    @arg_validation_decos.is_builtin_string(arg_position_index=5, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        dict_: Dictionary[Any, _DictValue],
        dict_value_type: Type[_DictValue],
        *,
        locals_: Optional[Dict[str, Any]] = None,
        globals_: Optional[Dict[str, Any]] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The loop implementation class for the `ap.Dictionary` values.

        Parameters
        ----------
        dict_ : Dictionary[Any, _DictValue]
            A dictionary to iterate.
        dict_value_type : Type[_DictValue]
            A dictionary value type. This interface accepts
            `InitializeWithBaseValueInterface` subclasses,
            such as the `Int`, `String`, or `Rectangle`.
        locals_ : Optional[Dict[str, Any]], optional
            Current scope's local variables. Set locals()
            value to this argument. If specified, this interface
            reverts all local scope VariableNameMixIn
            variables (like Int, Sprite) at the end of a with-statement
            scope. This setting is useful when you don't want to
            update each variable.
        globals_ : Optional[Dict[str, Any]], optional
            Current scope's global variables. Set globals()
            value to this argument. This setting works
            the same way as the locals_ argument.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - ForDictValues class
            - https://simon-ritchie.github.io/apysc/en/for_dict_values.html

        References
        ----------
        >>> import apysc as ap

        >>> _ = ap.Stage(
        ...     stage_width=250,
        ...     stage_height=150,
        ...     background_color=ap.Color("#333"),
        ...     stage_elem_id="stage",
        ... )
        >>> dict_: ap.Dictionary[str, ap.Number] = ap.Dictionary(
        ...     {"a": ap.Number(50), "b": ap.Number(150)},
        ... )
        >>> with ap.ForDictValues(dict_=dict_, dict_value_type=ap.Number) as value:
        ...     _ = ap.Rectangle(
        ...         x=value, y=50, width=50, height=50, fill_color=ap.Color("#0af")
        ...     )
        ...
        """
        self._initialize_locals_and_globals(locals_, globals_=globals_)
        self._dict = dict_
        self._dict_value_type = dict_value_type
        self._variable_name_suffix = variable_name_suffix
        self._indent = Indent()

    @final
    def _get_last_scope(self) -> LastScope:
        """
        Get a target last scope value.

        Returns
        -------
        last_scope : LastScope
            A target last scope.
        """
        return LastScope.FOR_DICT_VALUES

    @final
    @add_debug_info_setting(module_name=__name__)
    def __enter__(self) -> _DictValue:
        """
        The entering method for the beginning of with-statement.

        Returns
        -------
        dict_value : _DictValue
            A dictionary value of iteration.
        """
        from apysc._expression import expression_data_util
        from apysc._loop import loop_count
        from apysc._type import revert_mixin
        from apysc._validation.variable_name_validation import (
            validate_variable_name_mixin_type,
        )

        loop_count.increment_current_loop_count()
        self._snapshot_name = revert_mixin.make_snapshots_of_each_scope_vars(
            locals_=self._locals, globals_=self._globals
        )
        dict_value: _DictValue = self._dict_value_type._initialize_with_base_value()
        dict_value_variable_name: str = validate_variable_name_mixin_type(
            instance=dict_value,
        ).variable_name

        dict_key: String = String("", variable_name_suffix=self._variable_name_suffix)
        expression: str = (
            f"for ({dict_key.variable_name} in {self._dict.variable_name}) {{"
            f"\n  {dict_value_variable_name} = {self._dict.variable_name}"
            f"[{dict_key.variable_name}];"
        )
        expression_data_util.append_js_expression(expression=expression)

        self._indent.__enter__()
        return dict_value



"""The mix-in class implementation for
the `_copy_color_if_default_value_specified` method.
"""

from typing_extensions import final

from apysc._color.color import Color
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class CopyColorIfDefaultValueSpecifiedMixIn:
    @final
    @arg_validation_decos.is_color(arg_position_index=1, optional=False)
    @arg_validation_decos.is_color(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _copy_color_if_default_value_specified(
        self,
        color: Color,
        default_color: Color,
    ) -> Color:
        """
        Copy the specified color if it is a default color.

        Parameters
        ----------
        color : Color
            A color.
        default_color : Color
            A default color.

        Returns
        -------
        color : Color
            Copied color if it is a default color.
        """
        if color._value.variable_name == default_color._value.variable_name:
            return color.copy()
        return color



"""The class implementation for the outlined `anchor` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAnchorOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `anchor` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M17,15l1.55,1.55c-0.96,1.69-3.33,3.04-5.55,3.37V11h3V9h-3V7.82C14.16,7.4,15,6.3,15,5c0-1.65-1.35-3-3-3S9,3.35,9,5 c0,1.3,0.84,2.4,2,2.82V9H8v2h3v8.92c-2.22-0.33-4.59-1.68-5.55-3.37L7,15l-4-3v3c0,3.88,4.92,7,9,7s9-3.12,9-7v-3L17,15z M12,4 c0.55,0,1,0.45,1,1s-0.45,1-1,1s-1-0.45-1-1S11.45,4,12,4z"/></g></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: int_and_number_to_hex.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Int and Number classes to_hex method": "# Int と Number クラスの to_hex メソッド",
    ##################################################
    "This page explains the `Int` and `Number` classes `to_hex` method.": "このページでは`Int`と`Number`クラスの`to_hex`メソッドについて説明します。",  # noqa
    ##################################################
    "## What method is this?": "## メソッド概要",
    ##################################################
    'The `to_hex` method returns a hexadecimal string (e.g., "1f") from an `ap.Int` or `ap.Number` value.': '`to_hex`メソッドは`ap.Int`や`ap.Number`型の値から16進数の文字列（例 : "1f"）を返却します。',  # noqa
    ##################################################
    "## Note for `ap.Number` value": "## `ap.Number`型の値における特記事項",
    ##################################################
    "If you use this method with an `ap.Number` value, this method ignores floating-point number.": "`ap.Number`の値でこのメソッドを使った場合浮動小数点数の値は無視されます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `to_hex` method requires no arguments.": "`to_hex`メソッドは引数の指定を必要としません。",
    ##################################################
    "This method returns a hexadecimal string (`ap.String` type value).": "このメソッドは16進数の文字列（`ap.String`型の値）を返却します。",  # noqa
    ##################################################
    '```py\n# runnable\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nint_value: ap.Int = ap.Int(28)\nhex_str: ap.String = int_value.to_hex()\nap.assert_equal(hex_str, "1c")\n\nnumber: ap.Number = ap.Number(28.5)\nhex_str = int_value.to_hex()\nap.assert_equal(hex_str, "1c")\n\nap.save_overall_html(dest_dir_path="int_and_number_to_hex_basic_usage/")\n```': '```py\n# runnable\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nint_value: ap.Int = ap.Int(28)\nhex_str: ap.String = int_value.to_hex()\nap.assert_equal(hex_str, "1c")\n\nnumber: ap.Number = ap.Number(28.5)\nhex_str = int_value.to_hex()\nap.assert_equal(hex_str, "1c")\n\nap.save_overall_html(dest_dir_path="int_and_number_to_hex_basic_usage/")\n```',  # noqa
    ##################################################
    "## to_hex method API": "## to_hex メソッドのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    'Get a hexadecimal string (e.g., "1f").<hr>': '16進数の文字列（例 : "1f"）を取得します。<hr>',
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `variable_name_suffix`: str, default ''": "- `variable_name_suffix`: str, default ''",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `hex_str`: String": "- `hex_str`: String",
    ##################################################
    '  - A hexadecimal string (e.g., "1f").': '  - 16進数の文字列（例 : "1f"）',
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "This method ignores floating point numbers.<hr>": "このメソッドは浮動小数点数を無視します。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> int_value: ap.Int = ap.Int(28)\n>>> hex_str: ap.String = int_value.to_hex()\n>>> hex_str\nString("1c")\n\n>>> number: ap.Number = ap.Number(28.5)\n>>> hex_str = int_value.to_hex()\n>>> hex_str\nString("1c")\n```': '```py\n>>> import apysc as ap\n>>> int_value: ap.Int = ap.Int(28)\n>>> hex_str: ap.String = int_value.to_hex()\n>>> hex_str\nString("1c")\n\n>>> number: ap.Number = ap.Number(28.5)\n>>> hex_str = int_value.to_hex()\n>>> hex_str\nString("1c")\n```',  # noqa
}



"""Class implementation for append line point mix-in.
"""

from typing import Union

from typing_extensions import final

from apysc._display.points_2d_mixin import Points2DMixIn
from apysc._display.points_var_name_mixin import PointsVarNameMixIn
from apysc._display.polygon_apply_current_points_mixin import (
    PolygonApplyCurrentPointsMixIn,
)
from apysc._display.set_x_and_y_with_minimum_point_interface_base import (
    SetXAndYWithMinimumPointInterfaceBase,
)
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class AppendLinePointMixIn(
    PolygonApplyCurrentPointsMixIn,
    Points2DMixIn,
    VariableNameSuffixMixIn,
    PointsVarNameMixIn,
):
    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def append_line_point(
        self,
        *,
        x: Union[float, Number],
        y: Union[float, Number],
    ) -> None:
        """
        Append line point at the end.

        Parameters
        ----------
        x : float, Number
            X-coordinate.
        y : float, Number
            Y-coordinate.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> polygon: ap.Polygon = sprite.graphics.draw_polygon(
        ...     points=[
        ...         ap.Point2D(x=0, y=0),
        ...         ap.Point2D(x=0, y=50),
        ...         ap.Point2D(x=50, y=50),
        ...     ]
        ... )
        >>> polygon.append_line_point(x=50, y=0)
        """
        from apysc._expression import expression_data_util
        from apysc._geom.point2d import Point2D
        from apysc._type import value_util

        if not hasattr(self, "_points_var_name"):
            raise AttributeError(
                "_points_var_name attribute is not set. Please add "
                "implementation to set that value when constructor "
                "or else."
            )
        suffix: str = self._get_attr_or_variable_name_suffix(value_identifier="points")
        point: Point2D = Point2D(x=x, y=y, variable_name_suffix=suffix)
        self.points.append(value=point)
        expression: str
        x_name: str = value_util.get_value_str_for_expression(value=x)
        y_name: str = value_util.get_value_str_for_expression(value=y)
        expression = f"{self._points_var_name}.push([{x_name}, {y_name}]);"
        expression_data_util.append_js_expression(expression=expression)
        self._apply_current_points()

        if isinstance(self, SetXAndYWithMinimumPointInterfaceBase):
            self._set_x_and_y_with_minimum_point()



"""The class implementation for the outlined `fiber_dvr` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFiberDvrOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `fiber_dvr` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M11.87 12.43l-1-3.43h-1.5l1.75 6h1.5l1.75-6h-1.5zM21 11.5v-1c0-.85-.65-1.5-1.5-1.5H16v6h1.5v-2h1.15l.85 2H21l-.9-2.1c.5-.25.9-.8.9-1.4zm-1.5 0h-2v-1h2v1zM6.5 9H3v6h3.5c.85 0 1.5-.65 1.5-1.5v-3C8 9.65 7.35 9 6.5 9zm0 4.5h-2v-3h2v3z"/></svg>'  # noqa



"""Common utilities for each validation interface.
"""


def append_additional_err_msg(
    *,
    err_msg: str,
    additional_err_msg: str,
) -> str:
    """
    Append an additional error message to a specified error message.

    Parameters
    ----------
    err_msg : str
        An error message.
    additional_err_msg : str
        An additional error message.

    Returns
    -------
    err_msg : str
        A result error message.
    """
    if not additional_err_msg:
        return err_msg
    err_msg += f"\n{additional_err_msg}"
    return err_msg



"""The class implementation for the outlined `alternate_email` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAlternateEmailOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `alternate_email` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 1.95c-5.52 0-10 4.48-10 10s4.48 10 10 10h5v-2h-5c-4.34 0-8-3.66-8-8s3.66-8 8-8 8 3.66 8 8v1.43c0 .79-.71 1.57-1.5 1.57s-1.5-.78-1.5-1.57v-1.43c0-2.76-2.24-5-5-5s-5 2.24-5 5 2.24 5 5 5c1.38 0 2.64-.56 3.54-1.47.65.89 1.77 1.47 2.96 1.47 1.97 0 3.5-1.6 3.5-3.57v-1.43c0-5.52-4.48-10-10-10zm0 13c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"/></svg>'  # noqa



"""The validation utilities for matrix data.
"""

from typing import Dict
from typing import List
from typing import Union

from apysc._type.array import Array
from apysc._type.dictionary import Dictionary
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String


def validate_matrix_list_data(
    *,
    matrix_list_data: List[Dict[str, Union[int, float, str]]],
    additional_err_msg: str = "",
) -> None:
    """
    Validate whether a specified matrix list data is a valid type or not.

    Parameters
    ----------
    matrix_list_data : List[Dict[str, Union[int, float, str]]]
        A matrix list data.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    TypeError
        - If a specified data type is not a list.
        - If values are not type of dict.
        - If a dictionary key's type is not str.
        - If a dictionary value's type is not int, float, or str.
    """
    from apysc._validation import validation_common_utils

    if not isinstance(matrix_list_data, list):
        err_msg: str = (
            f"A specified data type is not a list: {type(matrix_list_data).__name__} "
            f"\n{matrix_list_data}"
        )
        err_msg = validation_common_utils.append_additional_err_msg(
            err_msg=err_msg,
            additional_err_msg=additional_err_msg,
        )
        raise TypeError(err_msg)

    for data in matrix_list_data:
        if not isinstance(data, dict):
            err_msg = (
                "A specified list value's type is not the dict: "
                f"{type(data).__name__}\n{data}"
            )
            err_msg = validation_common_utils.append_additional_err_msg(
                err_msg=err_msg,
                additional_err_msg=additional_err_msg,
            )
            raise TypeError(err_msg)
        for key, value in data.items():
            if not isinstance(key, str):
                err_msg = (
                    "A specified dict key type is not the str: "
                    f"{type(key).__name__}\n{key}"
                )
                err_msg = validation_common_utils.append_additional_err_msg(
                    err_msg=err_msg,
                    additional_err_msg=additional_err_msg,
                )
                raise TypeError(err_msg)
            if not isinstance(value, (int, float, str)):
                err_msg = (
                    "A specified dict value type is not int, float, or str: "
                    f"{type(value).__name__}\n{value}"
                )
                err_msg = validation_common_utils.append_additional_err_msg(
                    err_msg=err_msg,
                    additional_err_msg=additional_err_msg,
                )
                raise TypeError(err_msg)


def validate_matrix_array_data(
    *,
    matrix_array_data: Array[Dictionary[String, Union[Int, Number, String]]],
    additional_err_msg: str,
) -> None:
    """
    Validate whether a specified matrix array data is a valid type or not.

    Parameters
    ----------
    matrix_array_data : Array[Dictionary[String, Union[Int, Number, String]]]
        A matrix array data.
    additional_err_msg : str
        An additional error message to display.

    Raises
    ------
    TypeError
        - If a specified data type is not `ap.Array`.
        - If values are not the type of the `ap.Dictionary`.
        - If a dictionary key's type is not `ap.String`.
        - If a dictionary value's type is not `ap.Int`, `ap.Number`, or `ap.String`.
    """
    from apysc._validation import validation_common_utils

    if not isinstance(matrix_array_data, Array):
        err_msg: str = (
            "A specified data type is not `ap.Array`: "
            f"{type(matrix_array_data).__name__}\n{matrix_array_data}"
        )
        err_msg = validation_common_utils.append_additional_err_msg(
            err_msg=err_msg,
            additional_err_msg=additional_err_msg,
        )
        raise TypeError(err_msg)

    for data in matrix_array_data._value:
        if not isinstance(data, Dictionary):
            err_msg = (
                "A specified `ap.Array` value's type is not the `ap.Dictionary`: "
                f"{type(data).__name__}\n{data}"
            )
            err_msg = validation_common_utils.append_additional_err_msg(
                err_msg=err_msg,
                additional_err_msg=additional_err_msg,
            )
            raise TypeError(err_msg)
        for key, value in data._value.items():
            if not isinstance(key, String):
                err_msg = (
                    "A specified dictionary key type is not the `ap.String`: "
                    f"{type(key).__name__}\n{key}"
                )
                err_msg = validation_common_utils.append_additional_err_msg(
                    err_msg=err_msg,
                    additional_err_msg=additional_err_msg,
                )
                raise TypeError(err_msg)
            if not isinstance(value, (Int, Number, String)):
                err_msg = (
                    "A specified dictionary value type is not `ap.Int`, "
                    f"`ap.Number`, or `ap.String`: {type(value).__name__}\n{value}"
                )
                err_msg = validation_common_utils.append_additional_err_msg(
                    err_msg=err_msg,
                    additional_err_msg=additional_err_msg,
                )
                raise TypeError(err_msg)



"""The class implementation for the `surround_sound` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSurroundSoundIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `surround_sound` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M20 4H4c-1.1 0-2 .9-2 2v12c0 1.1.9 2 2 2h16c1.1 0 2-.9 2-2V6c0-1.1-.9-2-2-2zM7.76 16.24l-1.41 1.41C4.78 16.1 4 14.05 4 12c0-2.05.78-4.1 2.34-5.66l1.41 1.41C6.59 8.93 6 10.46 6 12s.59 3.07 1.76 4.24zM12 16c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4zm5.66 1.66l-1.41-1.41C17.41 15.07 18 13.54 18 12s-.59-3.07-1.76-4.24l1.41-1.41C19.22 7.9 20 9.95 20 12c0 2.05-.78 4.1-2.34 5.66zM12 10c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: text_fill_color.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Text fill_color property": "# テキストの fill_color 属性",
    ##################################################
    "This page explains the text-related `fill_color` property.": "このページではテキスト関係の`fill_color`属性について説明します。",  # noqa
    ##################################################
    "## What property is this?": "## 属性の概要",
    ##################################################
    "The `fill_color` property interface updates or gets the instance's fill color.": "`fill_color`属性のインターフェイスでは塗りの色の値を更新したり取得したりすることができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter interface becomes a `Color` value, and the setter one also requires a `Color` value.": "getterのインターフェイスは`Color`型の値になります。setterのインターフェイスも`Color`型の値の指定が必要になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=300,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=250,\n    font_size=16,\n    x=25,\n    y=25,\n)\nmulti_line_text.fill_color = ap.Colors.CYAN_00AAFF\nap.assert_equal(multi_line_text.fill_color, ap.Colors.CYAN_00AAFF)\nap.save_overall_html(dest_dir_path="text_fill_color_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=300,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=250,\n    font_size=16,\n    x=25,\n    y=25,\n)\nmulti_line_text.fill_color = ap.Colors.CYAN_00AAFF\nap.assert_equal(multi_line_text.fill_color, ap.Colors.CYAN_00AAFF)\nap.save_overall_html(dest_dir_path="text_fill_color_basic_usage/")\n```',  # noqa
    ##################################################
    "## fill_color property API": "## fill_color 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a text's fill color.<hr>": "テキストの塗りの色を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `color`: Color": "- `color`: Color",
    ##################################################
    "  - A text color.": "  - テキストの色。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=300,\n...     stage_height=100,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=300,\n...     font_size=16,\n...     x=20,\n...     y=20,\n... )\n>>> multi_line_text.fill_color = ap.Colors.CYAN_00AAFF\n>>> assert multi_line_text.fill_color == ap.Colors.CYAN_00AAFF\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=300,\n...     stage_height=100,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=300,\n...     font_size=16,\n...     x=20,\n...     y=20,\n... )\n>>> multi_line_text.fill_color = ap.Colors.CYAN_00AAFF\n>>> assert multi_line_text.fill_color == ap.Colors.CYAN_00AAFF\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [MultiLineText class](https://simon-ritchie.github.io/apysc/en/multi_line_text.html)": "- [MultiLineText クラス](https://simon-ritchie.github.io/apysc/jp/jp_multi_line_text.html)",  # noqa
}



"""The class implementation for the `2k` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class Material2KIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `2k` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19 3H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-9 9.5H8v1h3V15H6.5v-2.5c0-.55.45-1 1-1h2v-1h-3V9H10c.55 0 1 .45 1 1v1.5c0 .55-.45 1-1 1zm8 2.5h-1.75l-1.75-2.25V15H13V9h1.5v2.25L16.25 9H18l-2.25 3L18 15z"/></svg>'  # noqa



"""The class implementation for the outlined `add_ic_call` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAddIcCallOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `add_ic_call` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M20 15.45c-1.25 0-2.45-.2-3.57-.57-.1-.03-.21-.05-.31-.05-.26 0-.51.1-.71.29l-2.2 2.2c-2.83-1.44-5.15-3.75-6.59-6.59l2.2-2.21c.28-.26.36-.65.25-1C8.7 6.4 8.5 5.2 8.5 3.95c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1 0 9.39 7.61 17 17 17 .55 0 1-.45 1-1v-3.5c0-.55-.45-1-1-1zM5.03 4.95h1.5c.07.88.22 1.75.45 2.58l-1.2 1.21c-.4-1.21-.66-2.47-.75-3.79zM19 18.92c-1.32-.09-2.6-.35-3.8-.76l1.2-1.2c.85.24 1.72.39 2.6.45v1.51zM18 5.95v-3h-2v3h-3v2h3v3h2v-3h3v-2z"/></svg>'  # noqa



"""The class implementation for the `restore_page` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRestorePageIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `restore_page` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M14 2H6c-1.1 0-1.99.9-1.99 2L4 20c0 1.1.89 2 1.99 2H18c1.1 0 2-.9 2-2V8l-6-6zm-2 16c-2.05 0-3.81-1.24-4.58-3h1.71c.63.9 1.68 1.5 2.87 1.5 1.93 0 3.5-1.57 3.5-3.5S13.93 9.5 12 9.5c-1.35 0-2.52.78-3.1 1.9l1.6 1.6h-4V9l1.3 1.3C8.69 8.92 10.23 8 12 8c2.76 0 5 2.24 5 5s-2.24 5-5 5z"/></svg>'  # noqa



"""The class implementation for the `lightbulb_outline` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLightbulbOutlineIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `lightbulb_outline` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><g><path d="M9,21c0,0.55,0.45,1,1,1h4c0.55,0,1-0.45,1-1v-1H9V21z M12,2C8.14,2,5,5.14,5,9c0,2.38,1.19,4.47,3,5.74V17 c0,0.55,0.45,1,1,1h6c0.55,0,1-0.45,1-1v-2.26c1.81-1.27,3-3.36,3-5.74C19,5.14,15.86,2,12,2z M14,13.7V16h-4v-2.3 C8.48,12.63,7,11.53,7,9c0-2.76,2.24-5,5-5s5,2.24,5,5C17,11.49,15.49,12.65,14,13.7z"/></g></g></g></svg>'  # noqa



"""The class implementation for the `donut_large` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialDonutLargeIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `donut_large` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><g><path d="M11,5.08V2C6,2.5,2,6.81,2,12s4,9.5,9,10v-3.08c-3-0.48-6-3.4-6-6.92S8,5.56,11,5.08z M18.97,11H22c-0.47-5-4-8.53-9-9 v3.08C16,5.51,18.54,8,18.97,11z M13,18.92V22c5-0.47,8.53-4,9-9h-3.03C18.54,16,16,18.49,13,18.92z"/></g></g></g></svg>'  # noqa



"""Class implementation for an SVG text-span.
"""

from typing import List
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._display.append_fill_alpha_attr_expression_mixin import (
    AppendFillAlphaAttrExpressionMixIn,
)
from apysc._display.append_fill_color_expression_mixin import (
    AppendFillColorAttrExpressionMixIn,
)
from apysc._display.append_line_alpha_attr_expression_mixin import (
    AppendLineAlphaAttrExpressionMixIn,
)
from apysc._display.append_line_color_attr_expression_mixin import (
    AppendLineColorAttrExpressionMixIn,
)
from apysc._display.append_line_thickness_attr_expression_mixin import (
    AppendLineThicknessAttrExpressionMixIn,
)
from apysc._display.css_mixin import CssMixIn
from apysc._display.fill_alpha_mixin import FillAlphaMixIn
from apysc._display.fill_color_mixin import FillColorMixIn
from apysc._display.get_bounds_mixin import GetBoundsMixIn
from apysc._display.graphics_base import GraphicsBase
from apysc._display.line_alpha_mixin import LineAlphaMixIn
from apysc._display.line_color_mixin import LineColorMixIn
from apysc._display.line_thickness_mixin import LineThicknessMixIn
from apysc._display.set_overflow_visible_setting_mixin import (
    SetOverflowVisibleSettingMixIn,
)
from apysc._display.svg_text_bold_mixin import SvgTextBoldMixIn
from apysc._display.svg_text_delta_x_mixin import SvgTextDeltaXMixIn
from apysc._display.svg_text_delta_y_mixin import SvgTextDeltaYMixIn
from apysc._display.svg_text_font_family_mixin import SvgTextFontFamilyMixIn
from apysc._display.svg_text_font_size_mixin import SvgTextFontSizeMixIn
from apysc._display.svg_text_italic_mixin import SvgTextItalicMixIn
from apysc._display.svg_text_set_bold_mixin import SvgTextSetBoldMixIn
from apysc._display.svg_text_set_delta_x_mixin import SvgTextSetDeltaXMixIn
from apysc._display.svg_text_set_delta_y_mixin import SvgTextSetDeltaYMixIn
from apysc._display.svg_text_set_font_family_mixin import SvgTextSetFontFamilyMixIn
from apysc._display.svg_text_set_font_size_value_mixin import (
    SvgTextSetFontSizeValueMixIn,
)
from apysc._display.svg_text_set_italic_mixin import SvgTextSetItalicMixIn
from apysc._display.svg_text_set_text_value_mixin import SvgTextSetTextValueMixIn
from apysc._display.svg_text_skip_fill_alpha_exp_appending_mixin import (
    SvgTextSkipFillAlphaExpAppendingMixIn,
)
from apysc._display.svg_text_skip_fill_color_exp_appending_mixin import (
    SvgTextSkipFillColorExpAppendingMixIn,
)
from apysc._display.svg_text_skip_line_alpha_exp_appending_mixin import (
    SvgTextSkipLineAlphaExpAppendingMixIn,
)
from apysc._display.svg_text_skip_line_color_exp_appending_mixin import (
    SvgTextSkipLineColorExpAppendingMixIn,
)
from apysc._display.svg_text_skip_line_thickness_exp_appending_mixin import (
    SvgTextSkipLineThicknessExpAppendingMixIn,
)
from apysc._display.svg_text_text_mixin import SvgTextTextMixIn
from apysc._display.use_hand_cursor_mixin import UseHandCursorMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._mask.svg_mask_mixin import SvgMaskMixIn
from apysc._type.array import Array
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.repr_interface import ReprInterface
from apysc._type.string import String
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class SvgTextSpan(
    ReprInterface,
    SetOverflowVisibleSettingMixIn,
    CssMixIn,
    GraphicsBase,
    FillColorMixIn,
    AppendFillColorAttrExpressionMixIn,
    SvgTextSkipFillColorExpAppendingMixIn,
    FillAlphaMixIn,
    AppendFillAlphaAttrExpressionMixIn,
    SvgTextSkipFillAlphaExpAppendingMixIn,
    LineColorMixIn,
    AppendLineColorAttrExpressionMixIn,
    SvgTextSkipLineColorExpAppendingMixIn,
    LineAlphaMixIn,
    AppendLineAlphaAttrExpressionMixIn,
    SvgTextSkipLineAlphaExpAppendingMixIn,
    AppendLineThicknessAttrExpressionMixIn,
    LineThicknessMixIn,
    SvgTextSkipLineThicknessExpAppendingMixIn,
    SvgTextTextMixIn,
    SvgTextSetTextValueMixIn,
    SvgTextFontFamilyMixIn,
    SvgTextSetFontFamilyMixIn,
    SvgTextFontSizeMixIn,
    SvgTextSetFontSizeValueMixIn,
    SvgTextItalicMixIn,
    SvgTextSetItalicMixIn,
    SvgTextBoldMixIn,
    SvgTextSetBoldMixIn,
    SvgTextDeltaXMixIn,
    SvgTextSetDeltaXMixIn,
    SvgTextDeltaYMixIn,
    SvgTextSetDeltaYMixIn,
    GetBoundsMixIn,
    UseHandCursorMixIn,
    VariableNameSuffixMixIn,
    InitializeWithBaseValueInterface,
    SvgMaskMixIn,
):
    """
    The class for an SVG text-span (the child class of `SvgText`).

    Notes
    -----
    - If style settings are `None`, its styles inherit parent
        style settings.

    References
    ----------
    - SvgText class
        - https://simon-ritchie.github.io/apysc/en/svg_text.html
    - SvgTextSpan class
        - https://simon-ritchie.github.io/apysc/en/svg_text_span.html
    """

    # text
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    # font_size
    @arg_validation_decos.is_integer(arg_position_index=2, optional=True)
    # font_family
    @arg_validation_decos.is_builtin_str_list_or_apysc_str_arr(
        arg_position_index=3, optional=True
    )
    # fill_color
    @arg_validation_decos.is_color(arg_position_index=4, optional=True)
    # fill_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=5, optional=True)
    # line_color
    @arg_validation_decos.is_color(arg_position_index=6, optional=True)
    # line_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=7, optional=True)
    # line_thickness
    @arg_validation_decos.is_integer(arg_position_index=8, optional=True)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=8, optional=True)
    # bold
    @arg_validation_decos.is_boolean(arg_position_index=9, optional=True)
    # italic
    @arg_validation_decos.is_boolean(arg_position_index=10, optional=True)
    # delta_x
    @arg_validation_decos.is_num(arg_position_index=11, optional=False)
    # delta_y
    @arg_validation_decos.is_num(arg_position_index=12, optional=False)
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=13, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        text: Union[str, String],
        font_size: Optional[Union[int, Int]] = None,
        font_family: Optional[Union[Array[String], List[str]]] = None,
        fill_color: Optional[Color] = None,
        fill_alpha: Optional[Union[float, Number]] = None,
        line_color: Optional[Color] = None,
        line_alpha: Optional[Union[float, Number]] = None,
        line_thickness: Optional[Union[int, Int]] = None,
        bold: Optional[Union[bool, Boolean]] = None,
        italic: Optional[Union[bool, Boolean]] = None,
        delta_x: Union[float, Number] = 0.0,
        delta_y: Union[float, Number] = 0.0,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The class for an SVG text-span (the child class of `SvgText`).

        Notes
        -----
        - If style settings are `None`, its styles inherit parent
            style settings.

        References
        ----------
        - SvgText class
            - https://simon-ritchie.github.io/apysc/en/svg_text.html
        - SvgTextSpan class
            - https://simon-ritchie.github.io/apysc/en/svg_text_span.html

        Parameters
        ----------
        text : Union[str, String]
            A text to use in this class.
        font_size : Optional[Union[int, Int]], optional
            A font-size setting.
        font_family : Optional[Union[Array[String], List[str]]], optional
            A font-family setting.
            Each string in an array needs to be a font name (e.g., `Times New Roman`).
        fill_color : Optional[Color], optional
            A fill-color setting.
        fill_alpha : Optional[Union[float, Number]], optional
            A fill-alpha setting.
        line_color : Optional[Color], optional
            A line-color setting.
        line_alpha : Optional[Union[float, Number]], optional
            A line-alpha setting.
        line_thickness : Optional[Union[int, Int]], optional
            A line-thickness (line-width) to set.
        bold : Optional[Union[bool, Boolean]], optional
            A boolean, whether this text is a bold style or not.
        italic : Optional[Union[bool, Boolean]], optional
            A boolean, whether a text is an italic style or not (normal).
        delta_x : Union[float, Number], optional
            A coordinate delta-x setting.
            Notes: This setting also changes a coordinate of subsequent
            `SvgTextSpan`'s instance.
        delta_y : Union[float, Number], optional
            A coordinate delta-y setting.
            Notes: This setting also changes a coordinate of subsequent
            `SvgTextSpan`'s instance.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"), stage_width=200, stage_height=50
        ... )
        >>> svg_text: ap.SvgText = ap.SvgText.create_with_svg_text_spans(
        ...     text_spans=[
        ...         ap.SvgTextSpan(text="Hello, "),
        ...         ap.SvgTextSpan(text="Hello, ", font_size=14),
        ...     ],
        ...     font_size=20,
        ...     fill_color=ap.Color("#0af"),
        ... )
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.SVG_TEXT_SPAN,
        )
        self.variable_name = variable_name
        self._variable_name_suffix = variable_name_suffix
        self._set_fill_color_expression_skipping_attr(fill_color=fill_color)
        self._set_fill_alpha_expression_skipping_attr(fill_alpha=fill_alpha)
        self._set_line_color_expression_skipping_attr(line_color=line_color)
        self._set_line_alpha_expression_skipping_attr(line_alpha=line_alpha)
        self._set_line_thickness_expression_skipping_attr(line_thickness=line_thickness)
        self._set_initial_basic_values(
            fill_color=_get_init_fill_color(fill_color=fill_color),
            fill_alpha=_get_init_fill_alpha_num(fill_alpha=fill_alpha),
            line_color=_get_init_line_color(line_color=line_color),
            line_thickness=_get_init_line_thickness_num(line_thickness=line_thickness),
            line_alpha=_get_init_line_alpha_num(line_alpha=line_alpha),
            line_cap=None,
            line_joints=None,
        )
        self._append_constructor_expression()
        self._set_text_value(text=text)
        self._set_font_size_value(font_size=font_size)
        self._set_font_family(font_family=font_family)
        self._set_bold(bold=bold)
        self._set_italic(italic=italic)
        self._set_delta_x(delta_x=delta_x)
        self._set_delta_y(delta_y=delta_y)

        super(SvgTextSpan, self).__init__(variable_name=variable_name)
        self._set_overflow_visible_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression string.
        """
        from apysc._display.svg_text import SvgText
        from apysc._display.svg_text_singleton_for_text_span import (
            SvgTextSingletonForTextSpan,
        )
        from apysc._expression import expression_data_util

        INDENT_NUM: int = 2
        parent: SvgText = SvgTextSingletonForTextSpan.get_instance()
        variable_name: str = self.variable_name
        expression: str = (
            f"var {variable_name} = {parent.variable_name}" "\n  .tspan()\n  .attr({"
        )
        expression = self._append_fill_color_attr_expression(
            expression=expression,
            indent_num=INDENT_NUM,
            skip_appending=self._skip_fill_color_expression_appending,
        )
        expression = self._append_fill_alpha_attr_expression(
            expression=expression,
            indent_num=INDENT_NUM,
            skip_appending=self._skip_fill_alpha_expression_appending,
        )
        expression = self._append_line_color_attr_expression(
            expression=expression,
            indent_num=INDENT_NUM,
            skip_appending=self._skip_line_color_expression_appending,
        )
        expression = self._append_line_thickness_attr_expression(
            expression=expression,
            indent_num=INDENT_NUM,
            skip_appending=self._skip_line_thickness_expression_appending,
        )
        expression = self._append_line_alpha_attr_expression(
            expression=expression,
            indent_num=INDENT_NUM,
            skip_appending=self._skip_line_alpha_expression_appending,
        )
        expression += "\n});"
        expression_data_util.append_js_expression(expression=expression)

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            This interface returns a type name and variable name
            (e.g., `SvgTextSpan("<variable_name>")`).
        """
        repr_str: str = f'{SvgTextSpan.__name__}("{self.variable_name}")'
        return repr_str

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "SvgTextSpan":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        svg_text_span : SvgTextSpan
            An initialized text span instance.
        """
        svg_text_span: SvgTextSpan = SvgTextSpan(text="")
        svg_text_span.visible = Boolean(False)
        return svg_text_span


def _get_init_line_color(*, line_color: Optional[Color]) -> Color:
    """
    Get an initial line-color.

    Parameters
    ----------
    line_color : Optional[Color]
        A line-color.

    Returns
    -------
    line_color_ : Color
        If the specified value is None, this interface returns the COLORLESS
        constant.
    """
    if line_color is None:
        return COLORLESS
    return line_color


def _get_init_fill_alpha_num(
    *, fill_alpha: Optional[Union[float, Number]]
) -> Union[float, Number]:
    """
    Get an initial fill-alpha number.

    Parameters
    ----------
    fill_alpha : Optional[Union[float, Number]]
        A fill-alpha setting.

    Returns
    -------
    fill_alpha_ : Union[float, Number]
        If the specified value is None, this interface returns a 1.0 number.
    """
    if fill_alpha is None:
        return 1.0
    return fill_alpha


def _get_init_fill_color(*, fill_color: Optional[Color]) -> Color:
    """
    Get an initial fill-color.

    Parameters
    ----------
    fill_color : Optional[Color]
        A fill-color setting.

    Returns
    -------
    fill_color_ : Color
        If the specified value is None, this interface returns the COLORLESS
        constant.
    """
    if fill_color is None:
        return COLORLESS
    return fill_color


def _get_init_line_alpha_num(
    *,
    line_alpha: Optional[Union[float, Number]],
) -> Union[float, Number]:
    """
    Get an initial line-alpha number.

    Parameters
    ----------
    line_alpha : Optional[Union[float, Number]]
        A line-alpha setting.

    Returns
    -------
    line_alpha_ : Union[float, Number]
        If the specified value is None, this interface returns a 1.0 number.
    """
    if line_alpha is None:
        return 1.0
    return line_alpha


def _get_init_line_thickness_num(
    *,
    line_thickness: Optional[Union[int, Int]],
) -> Union[int, Int]:
    """
    Get an initial line-thickness (line-width) number.

    Parameters
    ----------
    line_thickness : Optional[Union[int, Int]]
        A line-thickness (line-width) setting.

    Returns
    -------
    line_thickness_ : Union[int, Int]
        If the specified value is None, this interface returns 1 number.
    """
    if line_thickness is None:
        return 1
    return line_thickness



"""The class implementation for the outlined `pending_actions` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPendingActionsOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `pending_actions` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M17,12c-2.76,0-5,2.24-5,5s2.24,5,5,5c2.76,0,5-2.24,5-5S19.76,12,17,12z M18.65,19.35l-2.15-2.15V14h1v2.79l1.85,1.85 L18.65,19.35z M18,3h-3.18C14.4,1.84,13.3,1,12,1S9.6,1.84,9.18,3H6C4.9,3,4,3.9,4,5v15c0,1.1,0.9,2,2,2h6.11 c-0.59-0.57-1.07-1.25-1.42-2H6V5h2v3h8V5h2v5.08c0.71,0.1,1.38,0.31,2,0.6V5C20,3.9,19.1,3,18,3z M12,5c-0.55,0-1-0.45-1-1 c0-0.55,0.45-1,1-1c0.55,0,1,0.45,1,1C13,4.55,12.55,5,12,5z"/></g></svg>'  # noqa



"""Implementations of the Rectangle class.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._display import graphics
from apysc._display.add_to_parent_mixin import AddToParentMixIn
from apysc._display.append_fill_alpha_attr_expression_mixin import (
    AppendFillAlphaAttrExpressionMixIn,
)
from apysc._display.append_fill_color_expression_mixin import (
    AppendFillColorAttrExpressionMixIn,
)
from apysc._display.append_line_alpha_attr_expression_mixin import (
    AppendLineAlphaAttrExpressionMixIn,
)
from apysc._display.append_line_cap_attr_expression_mixin import (
    AppendLineCapAttrExpressionMixIn,
)
from apysc._display.append_line_color_attr_expression_mixin import (
    AppendLineColorAttrExpressionMixIn,
)
from apysc._display.append_line_joints_attr_expression_mixin import (
    AppendLineJointsAttrExpressionMixIn,
)
from apysc._display.append_line_thickness_attr_expression_mixin import (
    AppendLineThicknessAttrExpressionMixIn,
)
from apysc._display.append_x_attr_expression_mixin import AppendXAttrExpressionMixIn
from apysc._display.append_y_attr_expression_mixin import AppendYAttrExpressionMixIn
from apysc._display.child_mixin import ChildMixIn
from apysc._display.css_mixin import CssMixIn
from apysc._display.ellipse_height_mixin import EllipseHeightMixIn
from apysc._display.ellipse_width_mixin import EllipseWidthMixIn
from apysc._display.fill_alpha_mixin import FillAlphaMixIn
from apysc._display.fill_color_mixin import FillColorMixIn
from apysc._display.flip_x_mixin import FlipXMixIn
from apysc._display.flip_y_mixin import FlipYMixIn
from apysc._display.get_bounds_mixin import GetBoundsMixIn
from apysc._display.graphics_base import GraphicsBase
from apysc._display.height_mixin import HeightMixIn
from apysc._display.line_alpha_mixin import LineAlphaMixIn
from apysc._display.line_caps import LineCaps
from apysc._display.line_color_mixin import LineColorMixIn
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.line_dash_dot_setting_mixin import LineDashDotSettingMixIn
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_dash_setting_mixin import LineDashSettingMixIn
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_dot_setting_mixin import LineDotSettingMixIn
from apysc._display.line_joints import LineJoints
from apysc._display.line_joints_mixin import LineJointsMixIn
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._display.line_round_dot_setting_mixin import LineRoundDotSettingMixIn
from apysc._display.rotation_around_center_mixin import RotationAroundCenterMixIn
from apysc._display.rotation_around_point_mixin import RotationAroundPointMixIn
from apysc._display.scale_x_from_center_mixin import ScaleXFromCenterMixIn
from apysc._display.scale_x_from_point_mixin import ScaleXFromPointMixIn
from apysc._display.scale_y_from_center_mixin import ScaleYFromCenterMixIn
from apysc._display.scale_y_from_point_mixin import ScaleYFromPointMixIn
from apysc._display.set_overflow_visible_setting_mixin import (
    SetOverflowVisibleSettingMixIn,
)
from apysc._display.skew_x_mixin import SkewXMixIn
from apysc._display.skew_y_mixin import SkewYMixIn
from apysc._display.use_hand_cursor_mixin import UseHandCursorMixIn
from apysc._display.width_mixin import WidthMixIn
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._mask.svg_mask_mixin import SvgMaskMixIn
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.repr_interface import ReprInterface
from apysc._type.string import String
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class Rectangle(
    ReprInterface,
    XMixIn,
    AppendXAttrExpressionMixIn,
    YMixIn,
    AppendYAttrExpressionMixIn,
    SetOverflowVisibleSettingMixIn,
    CssMixIn,
    GraphicsBase,
    RotationAroundCenterMixIn,
    RotationAroundPointMixIn,
    ScaleXFromCenterMixIn,
    ScaleYFromCenterMixIn,
    ScaleXFromPointMixIn,
    ScaleYFromPointMixIn,
    FlipXMixIn,
    FlipYMixIn,
    SkewXMixIn,
    SkewYMixIn,
    WidthMixIn,
    HeightMixIn,
    EllipseWidthMixIn,
    EllipseHeightMixIn,
    FillColorMixIn,
    AppendFillColorAttrExpressionMixIn,
    FillAlphaMixIn,
    AppendFillAlphaAttrExpressionMixIn,
    LineColorMixIn,
    AppendLineColorAttrExpressionMixIn,
    LineAlphaMixIn,
    AppendLineAlphaAttrExpressionMixIn,
    AppendLineThicknessAttrExpressionMixIn,
    AppendLineCapAttrExpressionMixIn,
    LineJointsMixIn,
    AppendLineJointsAttrExpressionMixIn,
    LineDotSettingMixIn,
    LineDashSettingMixIn,
    LineRoundDotSettingMixIn,
    LineDashDotSettingMixIn,
    GetBoundsMixIn,
    UseHandCursorMixIn,
    VariableNameSuffixMixIn,
    InitializeWithBaseValueInterface,
    AddToParentMixIn,
    SvgMaskMixIn,
):
    """
    The rectangle vector graphics class.

    References
    ----------
    - Rectangle class
        - https://simon-ritchie.github.io/apysc/en/rectangle.html
    - Graphics draw_rect interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_rect.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=100, height=75
    ... )
    >>> rectangle.x
    Number(50.0)

    >>> rectangle.y
    Number(50.0)

    >>> rectangle.width
    Int(100)

    >>> rectangle.height
    Int(75)

    >>> rectangle.fill_color
    Color("#00aaff")
    """

    # self
    @arg_validation_decos.multiple_line_settings_are_not_set(arg_position_index=0)
    # x
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    # y
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    # width
    @arg_validation_decos.is_integer(arg_position_index=3, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=3, optional=False)
    # height
    @arg_validation_decos.is_integer(arg_position_index=4, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=4, optional=False)
    # ellipse_width
    @arg_validation_decos.is_integer(arg_position_index=5, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=5, optional=False)
    # ellipse_height
    @arg_validation_decos.is_integer(arg_position_index=6, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=6, optional=False)
    # fill_color
    @arg_validation_decos.is_color(arg_position_index=7, optional=False)
    # fill_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=8, optional=False)
    # line_color
    @arg_validation_decos.is_color(arg_position_index=9, optional=False)
    # line_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=10, optional=False)
    # line_thickness
    @arg_validation_decos.is_integer(arg_position_index=11, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=11, optional=False)
    # line_cap
    @arg_validation_decos.is_line_cap(arg_position_index=12, optional=True)
    # line_joints
    @arg_validation_decos.are_line_joints(arg_position_index=13, optional=True)
    # line_dot_setting
    @arg_validation_decos.is_line_dot_setting(arg_position_index=14)
    # line_dash_setting
    @arg_validation_decos.is_line_dash_setting(arg_position_index=15)
    # line_round_dot_setting
    @arg_validation_decos.is_line_round_dot_setting(arg_position_index=16)
    # line_dash_dot_setting
    @arg_validation_decos.is_line_dash_dot_setting(arg_position_index=17)
    # parent
    @arg_validation_decos.is_display_object_container(
        arg_position_index=18, optional=True
    )
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=19, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        x: Union[float, Number],
        y: Union[float, Number],
        width: Union[int, Int],
        height: Union[int, Int],
        ellipse_width: Union[int, Int] = 0,
        ellipse_height: Union[int, Int] = 0,
        fill_color: Color = COLORLESS,
        fill_alpha: Union[float, Number] = 1.0,
        line_color: Color = COLORLESS,
        line_alpha: Union[float, Number] = 1.0,
        line_thickness: Union[int, Int] = 1,
        line_cap: Optional[Union[String, LineCaps]] = None,
        line_joints: Optional[Union[String, LineJoints]] = None,
        line_dot_setting: Optional[LineDotSetting] = None,
        line_dash_setting: Optional[LineDashSetting] = None,
        line_round_dot_setting: Optional[LineRoundDotSetting] = None,
        line_dash_dot_setting: Optional[LineDashDotSetting] = None,
        parent: Optional[ChildMixIn] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        Create a rectangle vector graphic.

        Parameters
        ----------
        x : float or Number
            X-coordinate to start drawing.
        y : float or Number
            Y-coordinate to start drawing.
        width : Int or int
            Rectangle width.
        height : Int or int
            Rectangle height.
        ellipse_width : int or Int
            Ellipse width.
        ellipse_height : int or Int
            Ellipse height.
        fill_color : Color, default COLORLESS
            A fill-color to set.
        fill_alpha : float or Number, default 1.0
            A fill-alpha to set.
        line_color : Color, default COLORLESS
            A line-color to set.
        line_alpha : float or Number, default 1.0
            A line-alpha to set.
        line_thickness : int or Int, default 1
            A line-thickness (line-width) to set.
        line_cap : String or LineCaps or None, default None
            A line-cap setting to set.
        line_joints : String or LineJoints or None, default None
            A line-joints setting to set.
        line_dot_setting : LineDotSetting or None, default None
            A dot setting to set.
        line_dash_setting : LineDashSetting or None, default None
            A dash setting to set.
        line_round_dot_setting : LineRoundDotSetting or None, default None
            A round-dot setting to set.
        line_dash_dot_setting : LineDashDotSetting or None, default None
            A dash-dot (1-dot chain) setting to set.
        parent : ChildMixIn or None, default None
            A parent instance to add this instance.
            If the specified value is None, this interface uses
            a stage instance.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - Rectangle class
            - https://simon-ritchie.github.io/apysc/en/rectangle.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=50, y=50, width=100, height=100, fill_color=ap.Color("#00aaff")
        ... )
        >>> rectangle.x
        Number(50.0)
        >>> rectangle.y
        Number(50.0)
        >>> rectangle.width
        Int(100)
        >>> rectangle.height
        Int(100)
        >>> rectangle.fill_color
        Color("#00aaff")
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._variable_name_suffix = variable_name_suffix
        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.RECTANGLE
        )
        self.variable_name = variable_name
        self._update_x_and_skip_appending_exp(x=x)
        self._update_y_and_skip_appending_exp(y=y)
        self._update_width_and_skip_appending_exp(value=width)
        self._update_height_and_skip_appending_exp(value=height)
        self._set_initial_basic_values(
            fill_color=fill_color,
            fill_alpha=fill_alpha,
            line_color=line_color,
            line_thickness=line_thickness,
            line_alpha=line_alpha,
            line_cap=line_cap,
            line_joints=line_joints,
        )
        self._append_constructor_expression()
        self._set_line_setting_if_not_none_value_exists(
            line_dot_setting=line_dot_setting,
            line_dash_setting=line_dash_setting,
            line_round_dot_setting=line_round_dot_setting,
            line_dash_dot_setting=line_dash_dot_setting,
        )
        self._set_ellipse_settings_if_values_are_not_zero(
            ellipse_width=ellipse_width, ellipse_height=ellipse_height
        )
        super(Rectangle, self).__init__(variable_name=variable_name)
        self._set_overflow_visible_setting()
        self._add_to_parent(parent=parent)

    @final
    def _set_ellipse_settings_if_values_are_not_zero(
        self, *, ellipse_width: Union[int, Int], ellipse_height: Union[int, Int]
    ) -> None:
        """
        Set ellipse-related settings if values are not zero.

        Parameters
        ----------
        ellipse_width : Union[int, Int]
            Ellipse width to set.
        ellipse_height : Union[int, Int]
            Ellipse height to set.
        """
        if isinstance(ellipse_width, int) and ellipse_width == 0:
            return
        if isinstance(ellipse_height, int) and ellipse_height == 0:
            return
        if isinstance(ellipse_width, Int) and ellipse_width._value == 0:
            return
        if isinstance(ellipse_height, Int) and ellipse_height._value == 0:
            return
        if isinstance(ellipse_width, int):
            ellipse_width = Int(ellipse_width)
        if isinstance(ellipse_height, int):
            ellipse_height = Int(ellipse_height)
        self.ellipse_width = ellipse_width
        self.ellipse_height = ellipse_height

    @classmethod
    @final
    def _create_with_graphics(
        cls,
        *,
        graphics: "graphics.Graphics",
        x: Union[float, Number],
        y: Union[float, Number],
        width: Union[int, Int],
        height: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> "Rectangle":
        """
        Create a rectangle instance with the instance of
        specified graphics..

        Parameters
        ----------
        graphics : Graphics
            Graphics instance to link this instance.
        x : float or Number
            X-coordinate to start drawing.
        y : float or Number
            Y-coordinate to start drawing.
        width : Int or int
            Rectangle width.
        height : Int or int
            Rectangle height.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        rectangle : Rectangle
            A created rectangle instance.
        """
        rectangle: Rectangle = Rectangle(
            x=x,
            y=y,
            width=width,
            height=height,
            fill_color=graphics._fill_color,
            fill_alpha=graphics._fill_alpha,
            line_color=graphics._line_color,
            line_alpha=graphics._line_alpha,
            line_thickness=graphics._line_thickness,
            line_cap=graphics._line_cap,
            line_joints=graphics._line_joints,
            line_dot_setting=graphics._line_dot_setting,
            line_dash_setting=graphics._line_dash_setting,
            line_round_dot_setting=graphics._line_round_dot_setting,
            line_dash_dot_setting=graphics._line_dash_dot_setting,
            parent=graphics,
            variable_name_suffix=variable_name_suffix,
        )
        return rectangle

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Rectangle("<variable_name>")`).
        """
        repr_str: str = f'{Rectangle.__name__}("{self.variable_name}")'
        return repr_str

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append constructor expression.
        """
        from apysc._display.stage import Stage
        from apysc._display.stage import get_stage
        from apysc._expression import expression_data_util

        INDENT_NUM: int = 2
        variable_name: str = self.variable_name
        stage: Stage = get_stage()
        expression: str = (
            f"var {variable_name} = {stage.variable_name}"
            f"\n  .rect({self.width.variable_name}, "
            f"{self.height.variable_name})"
            "\n  .attr({"
        )
        expression = self._append_fill_color_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_fill_alpha_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_color_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_thickness_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_alpha_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_cap_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_joints_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_x_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_y_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression += "\n  });"
        expression_data_util.append_js_expression(expression=expression)

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "Rectangle":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        rectangle : Rectangle
            An initialized rectangle instance.
        """
        from apysc._type.boolean import Boolean

        rectangle: Rectangle = Rectangle(x=-2, y=-2, width=1, height=1)
        rectangle.visible = Boolean(False)
        return rectangle



"""The class implementation for the outlined `how_to_vote` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialHowToVoteOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `how_to_vote` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M18 13h-.68l-2 2h1.91L19 17H5l1.78-2h2.05l-2-2H6l-3 3v4c0 1.1.89 2 1.99 2H19c1.1 0 2-.89 2-2v-4l-3-3zm1 7H5v-1h14v1zm-7.66-4.98c.39.39 1.02.39 1.41 0l6.36-6.36c.39-.39.39-1.02 0-1.41L14.16 2.3c-.38-.4-1.01-.4-1.4-.01L6.39 8.66c-.39.39-.39 1.02 0 1.41l4.95 4.95zm2.12-10.61L17 7.95l-4.95 4.95-3.54-3.54 4.95-4.95z"/></svg>'  # noqa



"""Class implementation for the SVG text's font-size mix-in.
"""

from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos


class SvgTextFontSizeMixIn(
    VariableNameMixIn,
    RevertMixIn,
):
    _font_size: int = 16

    @property
    @add_debug_info_setting(module_name=__name__)
    def font_size(self) -> Int:
        """
        Get a current font-size setting.

        Returns
        -------
        font_size : Int
            A current font-size setting.
        """
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        suffix: str = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="font_size",
        )
        font_size: Int = Int(self._font_size, variable_name_suffix=suffix)
        self._append_font_size_getter_expression(font_size=font_size)
        return font_size

    @font_size.setter
    @arg_validation_decos.num_is_gte_zero(arg_position_index=1, optional=False)
    @arg_validation_decos.is_apysc_integer(arg_position_index=1)
    def font_size(self, value: Int) -> None:
        """
        Set a font-size setting.

        Parameters
        ----------
        value : Int
            A font-size setting.
        """
        from apysc._expression import expression_data_util

        self._font_size = value._value
        expression: str = f'{self.variable_name}.font("size", {value.variable_name});'
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_font_size_getter_expression(self, *, font_size: Int) -> None:
        """
        Append a font-size getter expression string.

        Parameters
        ----------
        font_size : Int
            A target font-size setting.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f'{font_size.variable_name} = {self.variable_name}.font("size");'
        )
        expression_data_util.append_js_expression(expression=expression)

    _font_size_snapshots: Optional[Dict[str, int]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_font_size_snapshots",
            value=self._font_size,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._font_size = self._get_snapshot_val_if_exists(
            current_value=self._font_size,
            snapshot_dict=self._font_size_snapshots,
            snapshot_name=snapshot_name,
        )



# pyright: reportIncompatibleMethodOverride=false

"""The color class implementation.
"""

from typing import Any
from typing import TypeVar

from apysc._color.blue_color_mixin import BlueColorMixIn
from apysc._color.color_copy_mixin import ColorCopyMixIn
from apysc._color.from_rgb_mixin import FromRgbMixIn
from apysc._color.green_color_mixin import GreenColorMixIn
from apysc._color.red_color_mixin import RedColorMixIn
from apysc._event.custom_event_mixin import CustomEventMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.string import String
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos

_StrOrString = TypeVar("_StrOrString", str, String)


class Color(
    CustomEventMixIn,
    ColorCopyMixIn["Color"],
    VariableNameMixIn,
    VariableNameSuffixMixIn,
    VariableNameSuffixAttrOrVarMixIn,
    FromRgbMixIn,
    RedColorMixIn,
    GreenColorMixIn,
    BlueColorMixIn,
):
    """
    The color class implementation.

    References
    ----------
    - Color class
        - https://simon-ritchie.github.io/apysc/en/color.html
    - Colors class
        - https://simon-ritchie.github.io/apysc/en/colors.html
    - MaterialDesignColors class
        - https://simon-ritchie.github.io/apysc/en/material_design_colors.html
    - COLORLESS constant
        - https://simon-ritchie.github.io/apysc/en/colorless.html
    - Color class from_rgb class method
        - https://simon-ritchie.github.io/apysc/en/color_from_rgb.html
    - Color class red_color property
        - https://simon-ritchie.github.io/apysc/en/red_color.html
    - Color class green_color property
        - https://simon-ritchie.github.io/apysc/en/green_color.html
    - Color class blue_color property
        - https://simon-ritchie.github.io/apysc/en/blue_color.html

    Examples
    --------
    >>> import apysc as ap
    >>> color: ap.Color = ap.Color("#0af")
    >>> color
    Color("#00aaff")

    >>> color = ap.Color("#ffffff")
    >>> color
    Color("#ffffff")
    """

    _value: String
    _variable_name_suffix: str

    @arg_validation_decos.is_hex_color_code_format(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        value: _StrOrString,
        *,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The color class implementation.

        Parameters
        ----------
        value : str or String
            A hexadecimal color code string (e.g., '#000000').
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - Color class
            - https://simon-ritchie.github.io/apysc/en/color.html
        - Colors class
            - https://simon-ritchie.github.io/apysc/en/colors.html
        - MaterialDesignColors class
            - https://simon-ritchie.github.io/apysc/en/material_design_colors.html
        - COLORLESS constant
            - https://simon-ritchie.github.io/apysc/en/colorless.html
        - Color class from_rgb class method
            - https://simon-ritchie.github.io/apysc/en/color_from_rgb.html
        - Color class red_color property
            - https://simon-ritchie.github.io/apysc/en/red_color.html
        - Color class green_color property
            - https://simon-ritchie.github.io/apysc/en/green_color.html
        - Color class blue_color property
            - https://simon-ritchie.github.io/apysc/en/blue_color.html

        Examples
        --------
        >>> import apysc as ap
        >>> color: ap.Color = ap.Color("#0af")
        >>> color
        Color("#00aaff")

        >>> color = ap.Color("#ffffff")
        >>> color
        Color("#ffffff")
        """
        from apysc._color import color_util

        value = color_util.complement_hex_color(hex_color_code=value)
        self._value = String(
            value=value,
            variable_name_suffix=variable_name_suffix,
        )
        self._variable_name_suffix = variable_name_suffix
        self._variable_name = self._value._variable_name

    @add_debug_info_setting(module_name=__name__)
    def __eq__(self, other: Any) -> Any:
        """
        Comparison method between two color values.

        Parameters
        ----------
        other : Color
            The other color value.

        Returns
        -------
        result : Boolean
            If the two color values are equal, this interface returns True.
        """
        from apysc._expression import expression_data_util

        if isinstance(other, str):
            raise TypeError(
                "The comparison between the `Color` class and `str` are not supported."
            )
        if isinstance(other, String):
            raise TypeError(
                "The comparison between the `Color` class and `String` "
                "are not supported."
            )
        if isinstance(other, Color):
            result: Boolean = Boolean(
                self._value._value == other._value._value,
                variable_name_suffix=self._variable_name_suffix,
            )
        else:
            result = Boolean(False, variable_name_suffix=self._variable_name_suffix)
        if isinstance(other, Color):
            expression: str = (
                f"{result.variable_name} = {self._value.variable_name}"
                f" === {other._value.variable_name};"
            )
            expression_data_util.append_js_expression(expression=expression)

        return result

    def __repr__(self) -> str:
        """
        Get a representation string of this instance.

        Returns
        -------
        repr_str : str
            Representation string of this instance.
        """
        if not hasattr(self, "_value"):
            return 'Color("")'
        return f'Color("{self._value._value}")'



"""The mix-in class implementation for the `_initialize_locals_and_globals`
method.
"""

from typing import Any
from typing import Dict
from typing import Optional

from typing_extensions import final


class InitializeLocalsAndGlobalsMixIn:
    _locals: Dict[str, Any]
    _globals: Dict[str, Any]

    @final
    def _initialize_locals_and_globals(
        self,
        locals_: Optional[Dict[str, Any]],
        globals_: Optional[Dict[str, Any]],
    ) -> None:
        """
        Initialize the `locals` and `globals` attributes.

        Parameters
        ----------
        locals_ : Optional[Dict[str, Any]]
            Current scope's local variables.
        globals_ : Optional[Dict[str, Any]]
            Current scope's global variables.
        """
        if locals_ is None:
            locals_ = {}
        if globals_ is None:
            globals_ = {}
        self._locals = locals_
        self._globals = globals_



"""This module is for the translation mapping data of the
following document:

Document file: get_bounds.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# get_bounds interface": "# get_bounds インターフェイス",
    ##################################################
    "This page explains the `get_bounds` method interface.": "このページでは`get_bounds`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `get_bounds` method returns an instance's bounding box (geometry data, such as the coordinates or size).": "`get_bounds`メソッドはインスタンスのバウンディングボックスのデータ（座標やサイズなどの幾何データ）を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `get_bounds` method returns a `RectangleGeom` instance.": "`get_bounds`メソッドでは`RectangleGeom`クラスのインスタンスを返却します。",  # noqa
    ##################################################
    "This method accepts an optional `target_coordinate_space_object` argument.": "このメソッドは省略可能な`target_coordinate_space_object`引数を受け付けます。",  # noqa
    ##################################################
    "If this argument is specified, the reference position of the returned value becomes the relative coordinates from the specified argument's `DisplayObject`.": "もしこの引数が指定された場合、返却値の基準となる座標は指定された引数の`DisplayObject`からの相対座標となります。",  # noqa
    ##################################################
    "This is useful when you want to get relative coordinates, such as the coordinates of the parent `DisplayObject`.": "この設定は親の`DisplayObject`の座標などの相対座標を取得したい場合などに役に立つことがあります。",  # noqa
    ##################################################
    "If you omit this argument, the reference position becomes the `Stage` coordinates (substantially becoming absolute coordinates).": "もしこの引数の指定を省略した場合、座標は`Stage`を基準とした座標となります（実質的に絶対座標となります）。",  # noqa
    ##################################################
    "Example of no argument:": "引数の指定を省略した場合の例 :",
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=500,\n    stage_height=440,\n    stage_elem_id="stage",\n)\ncircle: ap.Circle = ap.Circle(\n    x=250,\n    y=220,\n    radius=150,\n    fill_color=ap.Color("#0af"),\n)\nbounding_box: ap.RectangleGeom = circle.get_bounds()\n\nbox_rectangle: ap.Rectangle = ap.Rectangle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    width=bounding_box.width,\n    height=bounding_box.height,\n    line_color=ap.Color("#aaa"),\n)\n\nfill_color: ap.Color = ap.Color("#fd63c3")\nleft_x_and_top_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    radius=10,\n    fill_color=fill_color,\n)\nleft_x_and_top_y_text: ap.SvgText = ap.SvgText(\n    text="left_x and top_y",\n    x=bounding_box.left_x,\n    y=bounding_box.top_y - 15,\n    fill_color=fill_color,\n)\n\nap.save_overall_html(dest_dir_path="get_bounds_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=500,\n    stage_height=440,\n    stage_elem_id="stage",\n)\ncircle: ap.Circle = ap.Circle(\n    x=250,\n    y=220,\n    radius=150,\n    fill_color=ap.Color("#0af"),\n)\nbounding_box: ap.RectangleGeom = circle.get_bounds()\n\nbox_rectangle: ap.Rectangle = ap.Rectangle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    width=bounding_box.width,\n    height=bounding_box.height,\n    line_color=ap.Color("#aaa"),\n)\n\nfill_color: ap.Color = ap.Color("#fd63c3")\nleft_x_and_top_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    radius=10,\n    fill_color=fill_color,\n)\nleft_x_and_top_y_text: ap.SvgText = ap.SvgText(\n    text="left_x and top_y",\n    x=bounding_box.left_x,\n    y=bounding_box.top_y - 15,\n    fill_color=fill_color,\n)\n\nap.save_overall_html(dest_dir_path="get_bounds_basic_usage_1/")\n```',  # noqa
    ##################################################
    "Example of using the `target_coordinate_space_object` argument:": "`target_coordinate_space_object`引数を指定した場合の例 :",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=500,\n    stage_height=440,\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.x = ap.Number(250)\nsprite.y = ap.Number(220)\ncircle: ap.Circle = ap.Circle(\n    x=0,\n    y=0,\n    radius=150,\n    fill_color=ap.Color("#0af"),\n    parent=sprite,\n)\nbounding_box: ap.RectangleGeom = circle.get_bounds(\n    target_coordinate_space_object=sprite,\n)\n\nbox_rectangle: ap.Rectangle = ap.Rectangle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    width=bounding_box.width,\n    height=bounding_box.height,\n    line_color=ap.Color("#aaa"),\n    parent=sprite,\n)\n\nap.save_overall_html(dest_dir_path="get_bounds_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=500,\n    stage_height=440,\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.x = ap.Number(250)\nsprite.y = ap.Number(220)\ncircle: ap.Circle = ap.Circle(\n    x=0,\n    y=0,\n    radius=150,\n    fill_color=ap.Color("#0af"),\n    parent=sprite,\n)\nbounding_box: ap.RectangleGeom = circle.get_bounds(\n    target_coordinate_space_object=sprite,\n)\n\nbox_rectangle: ap.Rectangle = ap.Rectangle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    width=bounding_box.width,\n    height=bounding_box.height,\n    line_color=ap.Color("#aaa"),\n    parent=sprite,\n)\n\nap.save_overall_html(dest_dir_path="get_bounds_basic_usage_2/")\n```',  # noqa
    ##################################################
    "## get_bounds method API": "## get_bounds メソッドのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an instance's bounding-box geometry data.<hr>": "インスタンスのバウンディングボックスの幾何データを取得します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `target_coordinate_space_object`: DisplayObject or None, default None": "- `target_coordinate_space_object`: DisplayObject or None, default None",  # noqa
    ##################################################
    "  - Target coordinate space object. If None is specified, then this method returns the bounding-box data based on the stage. If a `DisplayObject` instance is specified, then this method returns the bounding-box data based on the specified object.": "  - 座標の基準となるオブジェクト。もしNoneが指定された場合、このメソッドはステージを基準としたバウンディングボックスのデータを返却します。もし`DisplayObject`のインスタンスが指定された場合、ごのメソッドは指定されたオブジェクトの座標を基準としたバウンディングボックスのデータを返却します。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `bounding_box`: RectangleGeom": "- `bounding_box`: RectangleGeom",
    ##################################################
    "  - An instance's bounding-box geometry data.": "  - インスタンスのバウンディングボックスの幾何データ。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"), stage_width=250, stage_height=350\n... )\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50,\n...     y=100,\n...     width=150,\n...     height=200,\n...     fill_color=ap.Color("#0af"),\n... )\n>>> bounding_box: ap.RectangleGeom = rectangle.get_bounds()\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"), stage_width=250, stage_height=350\n... )\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50,\n...     y=100,\n...     width=150,\n...     height=200,\n...     fill_color=ap.Color("#0af"),\n... )\n>>> bounding_box: ap.RectangleGeom = rectangle.get_bounds()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [RectangleGeom class](https://simon-ritchie.github.io/apysc/en/rectangle_geom.html)": "- [RectangleGeom クラス](https://simon-ritchie.github.io/apysc/jp/jp_rectangle_geom.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: timedelta_days.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# TimeDelta class days interface": "# TimeDelta クラスの days インターフェイス",
    ##################################################
    "This page explains the `TimeDelta` class's `days` property interface.": "このページでは`TimeDelta`クラスの`days`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `days` property returns the number of days between two `DateTime` instances.": "`days`属性は2つの`DateTime`クラスのインスタンス間の日数値を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `days`' type is the apysc's `Int` and truncates a fraction value.": "`days`属性の値の型はapyscの`Int`型となり、且つその値の小数点数は切り捨てられます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndatetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\ndatetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\ntimedelta_: ap.TimeDelta = datetime_1 - datetime_2\ndays: ap.Int = timedelta_.days\nassert days == 2\n\ndatetime_3: ap.DateTime = ap.DateTime(2022, 12, 7)\ndatetime_4: ap.DateTime = ap.DateTime(2022, 12, 5, hour=10)\ntimedelta_ = datetime_3 - datetime_4\ndays = timedelta_.days\nassert days == 1\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndatetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\ndatetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\ntimedelta_: ap.TimeDelta = datetime_1 - datetime_2\ndays: ap.Int = timedelta_.days\nassert days == 2\n\ndatetime_3: ap.DateTime = ap.DateTime(2022, 12, 7)\ndatetime_4: ap.DateTime = ap.DateTime(2022, 12, 5, hour=10)\ntimedelta_ = datetime_3 - datetime_4\ndays = timedelta_.days\nassert days == 1\n```",  # noqa
    ##################################################
    "## days property API": "## days 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get days in the duration.<hr>": "時間の間隔値の日数を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `days`: Int": "- `days`: Int",
    ##################################################
    "  - Days value. This interface ignores a fraction.": "  - 日数値。小数点数の分は無視されます。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> datetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\n>>> datetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\n>>> timedelta_: ap.TimeDelta = datetime_1 - datetime_2\n>>> timedelta_.days\nInt(2)\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> datetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\n>>> datetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\n>>> timedelta_: ap.TimeDelta = datetime_1 - datetime_2\n>>> timedelta_.days\nInt(2)\n```",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [TimeDelta class](https://simon-ritchie.github.io/apysc/en/timedelta.html)": "- [TimeDelta クラス](https://simon-ritchie.github.io/apysc/jp/jp_timedelta.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: display_object_get_and_set_css.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DisplayObject class get_css and set_css interfaces": "# DisplayObject クラスの get_css と set_css メソッドのインターフェイス",  # noqa
    ##################################################
    "This page will explain the `DisplayObject` class `get_css` and `set_css` method interfaces.": "このページでは`DisplayObject`クラスの`get_css`と`set_css`の各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `get_css` method will return a CSS string, and the `set_css` method will set the CSS setting to a `DisplayObject` instance.": "`get_css`メソッドは`DisplayObject`のインスタンスに設定されている特定のCSSの文字列を返却し、`set_css`メソッドは特定のCSSの値を設定します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Each interface requires the `name` argument as the CSS name. In addition, the `set_css` method interface also requires the `value` argument as the CSS value string.": "各インターフェイスはCSS名としての`name`引数の指定を必要とします。加えて、`set_css`メソッドではCSSの値の文字列としての`value`引数が必要になります。",  # noqa
    ##################################################
    "The following example sets the `none` value to the `display` CSS if the current CSS value is the default (blank string, `''`). Otherwise, revert the value to default (`Else` case) by the timer event (ticks every second).": "以下のコード例では1秒ごとのタイマーでCSSの`display`の値がもしデフォルトの空文字になっていれば`none`の値を設定しています。デフォルト値以外の値になっていればデフォルトの値へと戻しています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _SpriteOptions(TypedDict):\n    sprite: ap.Sprite\n\n\ndef on_timer(e: ap.TimerEvent, options: _SpriteOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = options["sprite"]\n    display_css_val: ap.String = sprite.get_css(name="display")\n    condition: ap.Boolean = display_css_val == "none"\n    with ap.If(condition):\n        sprite.set_css(name="display", value="")\n    with ap.Else():\n        sprite.set_css(name="display", value="none")\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _SpriteOptions = {"sprite": sprite}\ntimer: ap.Timer = ap.Timer(handler=on_timer, delay=1000, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="display_object_get_and_set_css_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _SpriteOptions(TypedDict):\n    sprite: ap.Sprite\n\n\ndef on_timer(e: ap.TimerEvent, options: _SpriteOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = options["sprite"]\n    display_css_val: ap.String = sprite.get_css(name="display")\n    condition: ap.Boolean = display_css_val == "none"\n    with ap.If(condition):\n        sprite.set_css(name="display", value="")\n    with ap.Else():\n        sprite.set_css(name="display", value="none")\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _SpriteOptions = {"sprite": sprite}\ntimer: ap.Timer = ap.Timer(handler=on_timer, delay=1000, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="display_object_get_and_set_css_basic_usage/")\n```',  # noqa
    ##################################################
    "## get_css API": "## get_css API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a CSS value string.<hr>": "CSSの設定値の文字列を取得します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `name`: str or String": "- `name`: str or String",
    ##################################################
    "  - CSS name (e.g., 'display').": "  - CSS名（例 : 'display'）。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `css`: ap.String": "- `css`: ap.String",
    ##################################################
    "  - CSS value.": "  - CSSの値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.5)\n>>> sprite.set_css(name="display", value="none")\n>>> sprite.get_css(name="display")\nString("none")\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.5)\n>>> sprite.set_css(name="display", value="none")\n>>> sprite.get_css(name="display")\nString("none")\n```',  # noqa
    ##################################################
    "## set_css API": "## set_css API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Set a specified value string to the CSS.<hr>": "CSSに指定された文字列の値を設定します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `name`: str or String": "- `name`: str or String",
    ##################################################
    "  - CSS name (e.g., 'display').": "  - CSS名（例 : 'display'）。",
    ##################################################
    "- `value`: str or String": "- `value`: str or String",
    ##################################################
    "  - A CSS value string (e.g., 'none').": "  - CSSの値の文字列（例 : 'none'）",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.5)\n>>> sprite.set_css(name="display", value="none")\n>>> sprite.get_css(name="display")\nString("none")\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.5)\n>>> sprite.set_css(name="display", value="none")\n>>> sprite.get_css(name="display")\nString("none")\n```',  # noqa
}



"""The class implementation for the outlined `qr_code` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialQrCodeOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `qr_code` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M3,11h8V3H3V11z M5,5h4v4H5V5z"/><path d="M3,21h8v-8H3V21z M5,15h4v4H5V15z"/><path d="M13,3v8h8V3H13z M19,9h-4V5h4V9z"/><rect height="2" width="2" x="19" y="19"/><rect height="2" width="2" x="13" y="13"/><rect height="2" width="2" x="15" y="15"/><rect height="2" width="2" x="13" y="17"/><rect height="2" width="2" x="15" y="19"/><rect height="2" width="2" x="17" y="17"/><rect height="2" width="2" x="17" y="13"/><rect height="2" width="2" x="19" y="15"/></g></g></svg>'  # noqa



"""The class implementation for the `voice_over_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialVoiceOverOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `voice_over_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12.99 9.18c0-.06.01-.12.01-.18 0-2.21-1.79-4-4-4-.06 0-.12.01-.18.01l4.17 4.17zm-6.1-3.56L4.27 3 3 4.27l2.62 2.62C5.23 7.5 5 8.22 5 9c0 2.21 1.79 4 4 4 .78 0 1.5-.23 2.11-.62L19.73 21 21 19.73l-8.62-8.62-5.49-5.49zM9 15c-2.67 0-8 1.34-8 4v2h16v-2c0-2.66-5.33-4-8-4zm7.76-9.64l-1.68 1.69c.84 1.18.84 2.71 0 3.89l1.68 1.69c2.02-2.02 2.02-5.07 0-7.27zM20.07 2l-1.63 1.63c2.77 3.02 2.77 7.56 0 10.74L20.07 16c3.9-3.89 3.91-9.95 0-14z"/></svg>'  # noqa
